% This demo file demonstrates simple usage of the optimization demodulation
% functions linearCvxDemod and logCvxDemod.  Both of these functions
% implement demodulation using convex programming, specifically with the
% Cvx Toolkit (http://www.stanford.edu/~boyd/cvx).  
%
% The linear method is also implemented as a quadratic programming problem, 
% and so requires either the Cvx package, or Matlab function quadprog.  
% The logarithmic method requires Cvx.
%
% For more information on each function, type 'help linearCvxDemod' or
% 'help logCvxDemod'
% 
% The first demonstration uses a simple cosine carrier, and so both methods
% work in their default settings.
%
% In the second demonstration, a harmonic carrier is used.  In this case,
% the linear method still works in the default form.  However, the
% logarithmic method is not ideal, and works much better with an increased
% spectral cost function weighting.
%
% Optimization Demodulation Package v1.0
% Gregory Sell 4/5/2010




%----------------------------------------------------------
%---------------  A Simple Carrier Example  ---------------
%----------------------------------------------------------

% Build signal to be demodulated
fs = 2000;
N = 1*fs;
t = ((0:N-1)/fs)';
mFreq1 = 2.2;
mFreq2 = 9.4;
cFreq = 200.7;
mP1 = 2*pi*rand;
mP2 = 2*pi*rand;
cP = 2*pi/exp(1);
m = .5*(cos(2*pi*mFreq1*t + mP1) + cos(2*pi*mFreq2*t + mP2));
m = .5 + .25*m;
m = hanning(N).*(.95*m/max(abs(m)));
c = cos(2*pi*cFreq*t + cP);
c = c/max(abs(c));
s = m.*c;

% Plot original signal
figure;subplot(311);plot(t,s);hold on;plot(t,m,'r-.');hold off;
axis([0 1 -1 1]);xlabel('Time (s)');ylabel('Amplitude');title('Simple Carrier Example');

% Demodulate signal
[mLin cLin] = linearCvxDemod(s,fs,'disp',1);
[mLog cLog] = logCvxDemod(s,fs,'disp',1);

% Scale logarithmic modulator for plotting
mLog = ((mLog.'*m)/(mLog.'*mLog))*mLog;

% Plot modulators
subplot(312);plot(t,mLin);hold on;plot(t,m,'r-.');hold off;
axis([0 1 0 1]);xlabel('Time (s)');ylabel('Amplitude');title('Linear Method Modulator');		
subplot(313);plot(t,mLog);hold on;plot(t,m,'r-.');hold off;
axis([0 1 0 1]);xlabel('Time (s)');ylabel('Amplitude');title('Logarithmic Method Modulator');



					
%----------------------------------------------------------
%--------------  A Harmonic Carrier Example  --------------
%----------------------------------------------------------

% Build harmonic signal
fs = 2000;
N = 1*fs;
t = ((0:N-1)/fs)';
mFreq1 = 2.2;
mFreq2 = 9.4;
mP1 = 2*pi*rand;
mP2 = 2*pi*rand;
m = cos(2*pi*mFreq1*t + mP1) + cos(2*pi*mFreq2*t + mP2);
m = .5 + .25*m;
m = hanning(N).*m;
m = .95*m/max(abs(m));
c = zeros(N,1);
cF0 = 90.7;
cf = cF0;
w = 1;
while cf<fs/2
    c = c+w*cos(2*pi*cf*t+2*pi*rand);
    cf = cf+cF0;
    w = -2/3*w;
end
c=c/max(abs(c));
s = m.*c;

% Plot original signal
figure;subplot(411);plot(t,s);hold on;plot(t,m,'r-.');hold off;
axis([0 1 -1 1]);xlabel('Time (s)');ylabel('Amplitude');title('Harmonic Carrier Example');

% Demodulate signal
[mLin cLin] = linearCvxDemod(s,fs,'disp',1);

% In log domain, demodulate with default settings, and then when adjusted weights
[mLog1 cLog] = logCvxDemod(s,fs,'disp',1);
[mLog10 cLog] = logCvxDemod(s,fs,'w',10,'disp',1);

% Scale logarithmic method modulators for plotting
mLog1 = ((mLog1.'*m)/(mLog1.'*mLog1))*mLog1;
mLog10 = ((mLog10.'*m)/(mLog10.'*mLog10))*mLog10;

% Plot modulators
subplot(412);plot(t,mLin);hold on;plot(t,m,'r-.');hold off;
axis([0 1 0 1]);xlabel('Time (s)');ylabel('Amplitude');title('Linear Method Modulator');
subplot(413);plot(t,mLog1);hold on;plot(t,m,'r-.');hold off;
axis([0 1 0 1]);xlabel('Time (s)');ylabel('Amplitude');title('Logarithmic Method Modulator (Default)');
subplot(414);plot(t,mLog10);hold on;plot(t,m,'r-.');hold off;
axis([0 1 0 1]);xlabel('Time (s)');ylabel('Amplitude');title('Logarithmic Method Modulator (Weight = 10)');
