function [m c] = linearCvxDemod(s,fs,varargin)

%
% [m c] = linearCvxDemod(s,fs)
% 
% This function performs a linear-domain demodulation of signal s with
% sampling rate fs.  The estimated modulator is returned as m and the
% corresponding carrier as c.
%
% [m c] = linearCvxDemod(s,fs,'flag',value,...)
%
% m, c, s, and fs are all as above.  Optionally, 'flag'/value pairs can be
% added to the arguments to change additional parameters of the
% optimization.
%
% 'disp' - Display progress: 1 - yes, 0 - no (default 0)
% 'windowLength' - Length, in seconds, of analysis windows (default 0.1 s)
% 'cvx' - Sets whether or not to use cvx, if available (default 1)
%
% Optimization Demodulation Package v1.0
% Gregory Sell 4/5/2010

dispValue = 0;
windowLength = .1;
cvxValue = 1;

if(nargin>2)
    if(mod(nargin,2)==1)
        disp('Error: Optional arguments should come in Flag/Value pairs');
        m = 0;c = 0;
        return;
    end
    
    n = 1;
    while(n<(nargin-2))
		flag = varargin{n};
		value = varargin{n+1};

        n = n+2;
        switch flag
            case 'disp'
                dispValue = value;
            case 'windowLength'
                windowLength = value;
			case 'cvx'
				cvxValue = value;
        end
    end
end

if(dispValue)
    cvx_quiet(false);
else
    cvx_quiet(true);
end

if max(abs(s))>1
	disp('Signal amplitude greater than 1.  Scaling down.');
	signalScale = 1/max(abs(s));
	s = s*signalScale;
else
	signalScale = 1;
end

N = length(s);

s = reshape(s,N,1);
M = ceil(windowLength*fs);
if mod(M,20),M = M+1;end
R = M/2;

sPad = [zeros(M-R,1);s;zeros(M,1)];

mPad = zeros(N+2*M-R,1);
win = sqrt(hanning(M,'periodic'));
for i=1:R:(N+M-R)

    if(dispValue)
        disp(' ');
        disp('*--------------------------');
        disp(['  Clip ',num2str((i-1)/R+1),' of ',num2str(floor((N+R)/R))]);
        disp('*--------------------------');
        disp(' ');
    end
     
    clip = win.*sPad(i:i+M-1);
    mTemp = oneLinearCvxDemod(clip,fs,cvxValue);
    
    mPad(i:i+M-1) = mPad(i:i+M-1)+win.*mTemp;
end

m = mPad(M-R+1:N+M-R)/signalScale;
c = s./m;