function [m c] = logCvxDemod(s,fs,varargin)

%
% [m c] = logCvxDemod(s,fs)
% NOTE: Requires cvx toolbox
% 
% This function performs a logarithmic-domain demodulation of signal s with
% sampling rate fs.  The estimated modulator is returned as m and the
% corresponding carrier as c.
%
% [m c] = logCvxDemod(s,fs,'flag',value,...)
%
% m, c, s, and fs are all as above.  Optionally, 'flag'/value pairs can be
% added to the arguments to change additional parameters of the
% optimization.
%
% 'w' - Optimization cost weight for spectral cost function (default 1)
% 'disp' - Display progress: 1 - yes, 0 - no (default)
% 'windowLength' - Length, in seconds, of analysis windows (default 0.1 s)
%
% Optimization Demodulation Package v1.0
% Gregory Sell 4/5/2010

if(exist('cvx_begin','file')==0)
    disp('Error: Logarithmic Demodulation requires cvx (availabe at http://www.stanford.edu/~boyd/cvx)');
    m = [];
    c = [];
    return;
end

dispValue = 0;
windowLength = .1;
weights = [1 1 .1 1];

if(nargin>2)
    if(mod(nargin,2)==1)
        disp('Error: Optional arguments should come in Flag/Value pairs');
        m = 0;c = 0;
        return;
    end
    
    n = 1;
    while(n<(nargin-2))
		flag = varargin{n};
		value = varargin{n+1};
        n = n+2;
        
        switch flag
            case 'disp'
                dispValue = value;
            case 'windowLength'
                windowLength = value;
            case 'w'
                weights(1:2) = value;
        end
    end
end

N = length(s);
M = ceil(windowLength*fs);
if mod(M,2),M = M+1;end
R = M/2;

sPad = [zeros(R,1);s;zeros(M,1)];

win = hann(M,'periodic');

mPad = zeros(N+M+R,1);

if(dispValue)
    cvx_quiet(false);
else
    cvx_quiet(true);
end

for i=1:R:(N+R)
    if(dispValue)
        disp(' ');
        disp('*--------------------------');
        disp(['  Clip ',num2str((i-1)/R+1),' of ',num2str(floor((N+R)/R))]);
        disp('*--------------------------');
        disp(' ');
    end
    
    clip = win.*sPad(i:i+M-1);
    mTemp = oneLogCvxDemod(hilbert(clip),fs,weights);
    
    mPad(i:i+M-1) = mPad(i:i+M-1) + mTemp;
end

m = mPad(R+1:R+N);
m = optimizeModSign(m);
c = s./m;